{{
=====================================================================
BeanieBadgeLCD.spin - An LCD driver using the UC1697V controller.
Copyright (c) 2011 by John Steven Denson (jazzed)
See MIT license terms below.
=====================================================================
}}

pub dummy

'===================================================
'
con ' LCD writable registers

    SET_ColAddrLSB      = $00
    SET_ColAddrMSB      = $10
    SET_TempComp        = $24
    SET_PanelLoad       = $28
    SET_PumpCtrl        = $2c
    SET_AdvProgCtrl     = $30               ' 2 byte command
    SET_ScrollLineLSB   = $40
    SET_ScrollLineMSB   = $50
    SET_RowAddrLSB      = $60
    SET_RowAddrMSB      = $70
    SET_VbiasPot        = $81               ' 2 byte command
    SET_PartDispCtrl    = $84
    SET_RamAddrCtrl     = $88
    SET_FixedLines      = $90               ' 2 byte command
    SET_LineRate        = $a0
    SET_AllPixelOn      = $a4
    SET_InvDisplay      = $a6
    SET_DisplayEna      = $a8
    SET_LcdMapCtrl      = $c0
    SET_NlineIvert      = $c8               ' 2 byte command
    SET_ColorPattern    = $d0
    SET_ColorMode       = $d4
    SET_ComScanFunc     = $d8
    SET_SystemReset     = $e2
    SET_NOP             = $e3
    SET_TestCtrl        = $e4               ' 2 byte command
    SET_LcdBiasRatio    = $e8
    SET_ComEnd          = $f1               ' 2 byte command
    SET_PartDispStart   = $f2               ' 2 byte command
    SET_PartDispEnd     = $f3               ' 2 byte command
    SET_WPStartColAddr  = $f4               ' 2 byte command
    SET_WPStartRowAddr  = $f5               ' 2 byte command
    SET_WPEndColAddr    = $f6               ' 2 byte command
    SET_WPEndRowAddr    = $f7               ' 2 byte command
    SET_WProgMode       = $f8
    SET_MtpOpCtrl       = $b8               ' 2 byte command
    SET_MtpWrMask       = $b9               ' 3 byte command
    SET_Vmtp1Pot        = $F4               ' 2 byte command
    SET_Vmtp2Pot        = $F5               ' 2 byte command
    SET_MtpWrTimer      = $F6               ' 2 byte command
    SET_MtpRdTimer      = $F7               ' 2 byte command

'===================================================
' screen geometry
'
con
    ROW_LENGTH          = 129
    COL_HEIGHT          = 129
    END_PAGE            = 129
    END_COL             = 127

    FONTWIDTH           = 5
    FONTHEIGHT          = 9
    FONTOFFSET          = 4


    COLS = 25
    ROWS = 14

    screensize = COLS * ROWS
    lastrow    = screensize - COLS

'===================================================
'
con
    LCDRD       = 19                        ' read pin
    LCDWR       = 18                        ' write pin
    LCDRS       = 8                         ' select - schematic name
    LCDCD       = LCDRS                     ' command/data is more appropriate

    LCDBK       = 17                        ' back light control
    CTRAMODE    = 4                         ' NCO mode
    FRQAVALUE   = $40000                    ' Fast enough for 12MHz RCFAST
    'FRQAVALUE   = $8000000                  ' Fast enough for 20KHz RCSLOW

'===================================================
' PASM _CMD_
con
    CMD_WRDAT   = 1     ' write display RAM
    CMD_WRCMD   = 2     ' write 1 byte command
    CMD_WRCMD2  = 3     ' write 2 byte command
    CMD_RDDATA  = 4     ' read display RAM
    CMD_RDSTAT  = 5     ' read status register
    CMD_SHOWCHR = 6     ' show text character
    CMD_BLOCK   = 7     ' fill rectangle
    CMD_SETPIXEL= 8     ' set pixel with color
    CMD_CLRCOLOR= 9     ' clear screen with color
    CMDMASK     = $F

    BUFLEN      = $20

'----------------------------------------------------------------------
' PASM
'
dat             org 0
entry
'long            0 [8]  ' would be used by a debugger stub

_tmp            mov     $, par
_cmdptr         mov     $, _tmp
_bit            add     _tmp, #4
_addrptr        mov     $, _tmp
_ndx            add     _tmp, #4
_lenptr         mov     $, _tmp
_len            add     _tmp, #4
_buffptr        mov     $, _tmp
_zero           mov     $, #0
_cmd            rdlong  fntarr, _addrptr
tptr            or      outa,   _LCDMASK
t0              or      dira,   _LCDMASK
t1              rdlong  fnttab, _addrptr
t2              add     fnttab, #4

                or      dira,   _LCDBKLIGHT
                mov     ctra,   _ctra       ' move back when done
                mov     frqa,   _frqa

'===================================================
' main command loop
'
cmdDone
                wrlong  _zero,_cmdptr

cmdLoop         rdlong  _cmd, _cmdptr       ' read command from pointer
                mov        t0,   #$1FF         ' let COG rest a little
                add        t0,   CNT           ' don't suck too much current
                waitcnt t0,   t0 nr
                tjz     _cmd, #cmdLoop      ' if_z is always 4 clocks, tjz is 8 for no jump

                'xor     outa,_LCDBKLIGHT    ' test code only
                'jmp     #$-2

                movs    :jump,#jumptable    ' get jumpbase
                and     _cmd,#CMDMASK       ' one of 7 states
                shl     _cmd,#1
                add     :jump,_cmd
                rdlong  tptr, _buffptr      ' data is passed by reference
:jump           jmp     #0-0

'===================================================
' each jump entry must have 2 and only 2 instructions
' =$0
jumptable       nop
                jmp     #cmdDone            ' 0 should never happen

'===================================================
' _writeDataCmd - write data to LCD
' =$1 GOOD
_writeDataCmd
                call    #_writeData
                jmp     #cmdDone

'===================================================
' _writeCmdCmd  - write cmd to LCD
' =$2 GOOD
_writeCmdCmd
                call    #_writeCmd
                jmp     #cmdDone

'===================================================
' _writeCmd2Cmd  - write cmd2 to LCD
' =$3 GOOD
_writeCmd2Cmd
                call    #_writeCmd2
                jmp     #cmdDone

'===================================================
' _readDataCmd - read data from LCD
' =$4 GOOD
_readDataCmd
                call    #_readData
                jmp     #cmdDone

'===================================================
' _readStatusCmd - read status from LCD
' =$5 GOOD
_readStatusCmd
                call    #_readStatus
                jmp     #cmdDone

'===================================================
' _showChar - print character to display
' =$6
_showCharCmd
                call    #showChar
                jmp     #cmdDone

'===================================================
' _fillBlock - fillBlock rectangle on display
' =$7 GOOD
_fillBlockCmd
                call    #_fillBlock
                jmp     #cmdDone

'===================================================
' _setPixel - setPixel on display
' =$8 GOOD
_SetPixelCmd
                call    #setPix
                jmp     #cmddone

'===================================================
' _clearScreen - clearScreen to a given color
' =$9 GOOD
_clearScreenCmd
                call    #_clearScreen
                jmp     #cmddone

'===================================================
' place-holder
' =$A
                nop
                jmp     #cmddone

'===================================================
' place-holder
' =$B
                nop
                jmp     #cmddone

'===================================================
' place-holder
' =$C
                nop
                jmp     #cmddone

'===================================================
' place-holder
' =$D
                nop
                jmp     #cmddone

'===================================================
' place-holder
' =$E
                nop
                jmp     #cmddone

'===================================================
' place-holder
' =$F
                nop
                jmp     #cmddone



'===================================================
' _sendColor - write color to LCD
'
_sendColor
                andn    outa, _LCDCD        ' toggle for bus phase data
                or      outa, _LCDCD        ' data mode
                or      dira, _LCDDAT
                movd    outa, c1            ' set data before write strobe
                call    #_writeStrobe
                movd    outa, c2            ' set data before write strobe
                call    #_writeStrobe
_sendColor_ret  ret

'===================================================
' _writeData - write data to LCD
'
_writeData
                rdlong  _len, _lenptr
                andn    outa, _LCDCD        ' toggle for bus phase data
                or      outa, _LCDCD        ' data mode
                or      dira, _LCDDAT
:loop
                rdbyte  t1,tptr
                movd    outa,t1             ' set data before write strobe
                call    #_writeStrobe
                add     tptr,#1
                djnz    _len, #:loop
_writeData_ret  ret

'===================================================
' _readData - read data to LCD
'
_readData
                andn    outa, _LCDCD        ' toggle for bus phase data
                or      outa, _LCDCD        ' data mode

                andn    dira, _LCDDAT
                call    #_dummyRead
                sub     tptr,#1             ' setup/hold about 100ns

                rdlong  _len, _lenptr
:loop
                andn    outa,_LCDRD         ' data read
                add     tptr,#1             ' setup 60ns max
                mov     t1,ina              ' data availble when low
                or      outa,_LCDRD
                shr     t1,#9
                wrbyte  t1,tptr
                djnz    _len, #:loop
_readData_ret   ret

'===================================================
' _readStatus - read Status to LCD
'
_readStatus
                or      outa, _LCDCD        ' toggle high for bus phase data
                andn    outa, _LCDCD        ' 0 for command mode

                andn    dira, _LCDDAT
                call    #_dummyRead

                sub     tptr,#1             ' setup/hold about 100ns
                mov     _len, #3            ' only 3 bytes are valid
:loop
                andn    outa,_LCDRD         ' data read
                add     tptr,#1             ' setup 60ns max
                mov     t1,ina              ' data availble when low
                or      outa,_LCDRD
                shr     t1,#9
                wrbyte  t1,tptr
                djnz    _len, #:loop
_readStatus_ret ret

'===================================================
' _writeCmd - write a command to LCD
'
_writeCmd 
                rdbyte  t0,_addrptr
                call    #_sendCmd
_writeCmd_ret   ret

_sendCmd
                or      outa, _LCDCD        ' toggle for bus phase data
                andn    outa, _LCDCD        ' 0 for command mode
                or      dira, _LCDDAT
                movd    outa,t0
                call    #_writeStrobe
_sendCmd_ret    ret


'===================================================
' _writeCmd2 - write 2 command bytes to LCD
'
_writeCmd2
                rdbyte  t0,_addrptr
                mov     tptr,_buffptr
                rdbyte  t1,tptr             ' data is passed by value
                call    #_sendCmd2
_writeCmd2_ret  ret

_sendCmd2 
                or      outa, _LCDCD        ' toggle for bus phase data
                andn    outa, _LCDCD        ' 0 for command mode
                or      dira, _LCDDAT
                movd    outa, t0
                call    #_writeStrobe
                movd    outa, t1
                call    #_writeStrobe
_sendCmd2_ret   ret

'===================================================
' _writeStrobe
'
_writeStrobe
                andn    outa,_LCDWR
                nop                         ' setup/hold
                or      outa,_LCDWR
_writeStrobe_ret ret

'===================================================
' _dummyRead - required for reading data and status
'
_dummyRead
                andn    outa,_LCDRD         ' dummy read
                nop                         ' setup/hold
                or      outa,_LCDRD
_dummyRead_ret  ret

'===================================================
' _setClolor
'
_setColor
                mov     c2, pix
                and     c2, #$ff
                mov     c1, pix
                shr     c1, #8
                and     c1, #$ff
_setColor_ret   ret

'===================================================
' _setField - sets the field x,y for rectangular objects
'
_setField
                mov     t0, x
                and     t0, #$f              ' position and mask x bits
                mov     t1, x
                shr     t1, #4
                and     t1, #$7
                or      t0, #SET_ColAddrLSB ' add command bits
                or      t1, #SET_ColAddrMSB
                call    #_sendCmd2

                mov     t0, y
                and     t0, #$f              ' position and mask y bits
                mov     t1, y
                shr     t1, #4
                and     t1, #$7
                or      t0, #SET_RowAddrLSB ' add command bits
                or      t1, #SET_RowAddrMSB
                call    #_sendCmd2

_setField_ret   ret

'===================================================
''cmdClearScreen
'' @param pix color to clear in bufptr[0]
''
_clearScreen
                rdlong  pix, _buffptr
                call    #_setColor

                mov     x, #0
                mov     y, #0
                mov     width, #END_PAGE
                mov     height,#END_PAGE
                call    #_setField

                mov     _len, PAGE_PIXELS
                shl     _len, #1
                andn    outa, _LCDCD        ' toggle for bus phase data
                or      outa, _LCDCD        ' data mode
                or      dira, _LCDDAT
clrloop
                movd    outa, c1            ' set data before write strobe
                andn    outa,_LCDWR         ' nop for setup/hold?
                or      outa,_LCDWR
                movd    outa, c2            ' set data before write strobe
                andn    outa,_LCDWR         ' nop for setup/hold?
                or      outa,_LCDWR
                djnz    _len, #clrloop
_clearScreen_ret    ret
  


'===================================================
'setPix
' same as cmdSetPixel except its called
' @param pix color to set in bufptr[0] ' long wide
' @param x xposition in bufptr[1]      ' long wide
' @param y xposition in bufptr[2]      ' long wide

setPix
                mov     _len, _buffptr
                rdlong  pix, _len
                add     _len, #4
                rdlong  x,   _len
                add     _len, #4
                rdlong  y,   _len

                ' x,y already set
                mov     width, x
                mov     height,y

                call    #_setField
                call    #_setColor
                call    #_sendColor
{
                andn        outa, _LCDCD        ' toggle for bus phase data
                or          outa, _LCDCD        ' data mode
                or          dira, _LCDDAT
                movd        outa, c1            ' set data before write strobe
                andn        outa,_LCDWR         ' nop for setup/hold?
                or          outa,_LCDWR
                movd        outa, c2            ' set data before write strobe
                andn        outa,_LCDWR         ' nop for setup/hold?
                or          outa,_LCDWR
'}
setPix_ret  ret

''==================================================
'' _fillBlock - display a filled rectangle on the LCD
''--------------------------------------------------
'' @param fcolor - foreground color        long buf[0]
'' @param mx - left position               long buf[1]
'' @param my - top position                long buf[2]
'' @param width - rect width               long buf[3]
'' @param height - rect height             long buf[4]
'' @param outline - draw outline width     long buf[5]
''
_fillBlock
                mov         tptr,  _buffptr
                rdlong      pix, tptr       ' buf 0
                add         tptr,  #4
                rdlong      x,   tptr       ' buf 1
                add         tptr,  #4
                rdlong      y,   tptr       ' buf 2
                add         tptr,  #4
                rdlong      width,tptr      ' buf 3
                add         tptr,  #4
                rdlong      height,tptr     ' buf 4
                add         tptr,  #4
                rdlong      outline,tptr    ' buf 5

                and         width,#$7f
                and         height,#$7f
                and         outline,#$7f    ' check for 0 outline

                tjnz        outline,#$+2
                call        #fillRect
                tjz         outline,#$+2
                call        #drawRect
_fillBlock_ret  ret


'===================================================
'fillRect
' @param pix color to clear in buf[0]
' @param x left                buf[1]
' @param y top                 buf[2]
' @param width                 buf[3]
' @param height                buf[4]
'
fillRect
                mov         twid, width
                mov         thit, height
                add         width, x
                add         height,y
                mov         ty, thit
fillRecthloop
                call        #_setField
                call        #_setColor
                mov         tx, twid
                add         tx, #1
                andn        outa, _LCDCD        ' toggle for bus phase data
                or          outa, _LCDCD        ' data mode
                or          dira, _LCDDAT
fillRectwLoop
'               call        #_sendColor
'{
                movd        outa, c1            ' set data before write strobe
                andn        outa,_LCDWR         ' nop for setup/hold?
                or          outa,_LCDWR
                movd        outa, c2            ' set data before write strobe
                andn        outa,_LCDWR         ' nop for setup/hold?
                or          outa,_LCDWR
'}
                djnz        tx, #fillRectwLoop
                add         y, #1
                djnz        ty, #fillRecthloop
fillRect_ret ret

{
fillRectSP
                mov         sx, x
                mov         thit,height
fillRectYloop
                mov         twid,width
fillRectXloop
                mov         tptr, _buffptr

                wrlong      pix,tptr
                add         tptr, #4
                wrlong      x, tptr
                add         tptr, #4
                wrlong      y, tptr
                add         tptr, #4
                call        #setPix

                add         x, #1
                djnz        twid,#fillRectXloop
                add         y, #1
                mov         x, sx
                djnz        thit,#fillRectYloop

fillRectSP_ret  ret
}

'===================================================
'drawRect
' @param pix color to clear in buf[0]
' @param x left                buf[1]
' @param y top                 buf[2]
' @param width                 buf[3]
' @param height                buf[4]
'
drawRect
                tjz         outline, drawRect_ret
                mov         twid, width
                mov         thit, height
                add         width, x
                add         height,y
                mov         sx, x               ' save x for right side
                mov         sy, y               ' save y for bottom side
  
                call        #_setField
                call        #_setColor

' draw top line with outline height
                mov         ty, outline
drawRectThloop
                mov         tx, twid
                add         tx, #1
drawRectTwLoop
                call        #_sendColor
                djnz        tx, #drawRectTwLoop
                add         y, #1
                djnz        ty, #drawRectThloop

                ' draw left side
                mov         y, sy
                mov         ty, thit
drawRect1hloop
                call        #_setField
                mov         tx, outline
drawRect1wLoop
                call        #_sendColor
                djnz        tx, #drawRect1wLoop
                add         y, #1
                djnz        ty, #drawRect1hloop

' draw right side
                mov         y, sy
                mov         x, sx
                add         x, twid
                sub         x, outline
                add         x, #1

                mov         ty, thit
drawRect2hloop
                call        #_setField
                mov         tx, outline
drawRect2wLoop
                call        #_sendColor
                djnz        tx, #drawRect2wLoop
                add         y, #1
                djnz        ty,#drawRect2hloop

                ' draw bottom line with outline height
                mov         y, sy
                add         y, thit
                sub         y, outline
                'sub         y, #1
                mov         x, sx
                call        #_setField
  
                mov         ty, outline
drawRectBhloop
                mov         tx, twid
                add         tx, #1
drawRectBwLoop
                call        #_sendColor
                djnz        tx, #drawRectBwLoop
                add         y, #1
                djnz        ty, #drawRectBhloop
drawRect_ret ret

'===================================================
'' multiply
'' @parm t1          term 1
'' @parm t2          term 2
'' @return rc        product
''
multiply
                mov         rc,#0
:loop
                shr         t1,#1   wc
                if_c  add   rc,t2
                shl         t2,#1
                tjnz        t1,#:loop
multiply_ret  ret

'===================================================
'
showChar
                mov         tptr,  _buffptr
                rdlong      pch, tptr
                add         tptr,  #4
                rdlong      fgd, tptr
                add         tptr,  #4
                rdlong      bgd, tptr
                add         tptr,  #4
                rdlong      x,   tptr
                add         tptr,  #4
                rdlong      y,   tptr

                mov         tptr,  fntarr
                rdword      tptr,  tptr
                shl         pch, #1
                add         tptr,  pch
                rdword      adr, tptr           ' get char addr
                add         tptr,  #2

                mov         t1, adr           ' the font is some standard made with photoshop
                mov         t2, #FONTHEIGHT   ' there are 45 bits, so we multiply by 9
                call        #multiply         ' and use div and mod to get good alignments

                mov         adr, rc
                shr         adr, #3 ' div 8

                ' at this point: lct = number of pixel lines, adr = char addr offset ptr

                mov         bit, #$80
                and         rc, #$7           ' mod 8
                shr         bit, rc           ' scan bits left to right starting at this position
                                ' subsequent bit positions start at $80
    
                rdword      fp,  fnttab       ' get font bit table pointer
                add         fp,  adr          ' char font base ptr
                mov         bits,#FONTWIDTH
                rdbyte      byt, fp           ' get first byte of char

                mov         sx,  x            ' save x
                mov         bcnt,FONT_PIXELS  ' show exactly Fnt W*H pixels
showCharBits
                sub         bcnt, #1
                call        #showp
                add         x,   #1           ' next bit position
                test        bit, #1 wz
                if_z  shr   bit, #1           ' if more bits shift mask
                if_z  jmp   #showCharSkipLd
                add         fp,  #1           ' inc ptr to get more bits
                rdbyte      byt, fp           ' get next byte of char
                mov         bit, #$80         ' else reset bit mask
showCharSkipLd
                sub         bits,#1 wz
                if_nz jmp   #showCharBits
                add         y,   #1           ' next line in char
                mov         x,   sx           ' reset x bit position
                mov         bits,#FONTWIDTH   ' number of columns = bits per row
                tjnz        bcnt,#showCharBits' show exactly 45 pixels
showCharDone  
showChar_ret  ret  

'---------------------------------
' part of show char
'
showp
                mov         tptr,  _buffptr           ' get arg passing base pointer
                test        byt, bit wz       ' if bit set
                if_nz wrlong fgd, tptr        '   save foreground color
                if_z  wrlong bgd, tptr        ' else save background
                if_z  tjz   bgd, showp_ret    ' if black bacground, no pixel
                add         tptr,  #4 
                wrlong      x,   tptr         ' save x pos
                add         tptr,  #4
                wrlong      y,   tptr         ' save y pos
                call        #setPix           ' set pixel
showp_ret     ret  


'----------------------------------------------------------------------

'-------------- LCD pins -------------
_LCDCD          long (1 << LCDCD)
_LCDRD          long (1 << LCDRD)
_LCDWR          long (1 << LCDWR)
_LCDBKLIGHT     long (1 << LCDBK)
_LCDDAT         long ($ff << 9)     ' all data bits are in DST slot.
_LCDMASK        long (1 << LCDRD) | (1 << LCDWR) | (1 << LCDCD)

'-------------- pixels   -------------
PAGE_PIXELS     long ((END_PAGE*END_PAGE)/2 - 1)
FONT_PIXELS     long (FONTWIDTH*FONTHEIGHT)

'-------------- counters -------------
_ctra           long (CTRAMODE << 26) | LCDBK
_frqa           long FRQAVALUE

'----------------------------------------------------------------------

pix             res 1
x               res 1
y               res 1
width           res 1
height          res 1
outline         res 1

c1              res 1
c2              res 1
rc              res 1
acc             res 1

sx              res 1
sy              res 1
tx              res 1
ty              res 1
twid            res 1
thit            res 1

pch             res 1
fgd             res 1
bit             res 1
byt             res 1
bgd             res 1
bits            res 1
bcnt            res 1

adr             res 1
fp              res 1

fntarr          res 1           ' font array ptr
fnttab          res 1           ' font table ptr

'----------------------------------------------------------------------

fit             $1ff

'obj fnt : "fontms5x9"

dat
    farr    long 0
    ftab    long 0

