/*
 * LCD.h
 */
#ifndef __LCD_H__
#define __LCD_H__

#if defined(__PROPELLER_GCC__)
#define HUB HUBDATA
#else
#define HUB
#endif

extern HUB int LCD_fgcolor;
extern unsigned int binary_UC1697a_LCD_dat_start;

#define binary_LCD_dat_start binary_UC1697a_LCD_dat_start

/*
 * ---------------------------------------------------
 * GameBaby 16-Bit Color Definitions
 *
 * use TRANSPARENT for showChar showString to print chars without background
 */
#define LCD_TRANSPARENT   0x0000    // 0

/*
 * ---------------------------------------------------
 * not really black, but better on screen ... 0 is TRANSPARENT
 */
#define LCD_BLACK         0x0840    // %00001_000010_00000 

/*
 * ---------------------------------------------------
 * some useful colors
 */
#define LCD_WHITE         0xffff    // %1111.1_111.111_1.1111
#define LCD_RED           0xf800    // %1111.1_000.000_0.0000
#define LCD_GREEN         0x07e0    // %0000.0_111.111_0.0000
#define LCD_DARK_GREEN    0x0200    // %0000.0_010.000_0.0000
#define LCD_BLUE          0x001f    // %0000.0_000.000_1.1111
#define LCD_CYAN          0xa51f    // %1010.0_101.000_1.1111
#define LCD_MAGENTA       0xd01f    // %1101.0_000.000_1.1111
#define LCD_YELLOW        0xffe4    // %1111.1_111.111_0.0100
#define LCD_BROWN         0xb084    // %1011.0_000.100_0.0100
#define LCD_ORANGE        0xfac0    // %1111.1_010.110_0.0000
#define LCD_PINK          0xfc1f    // %1111.1_100.000_1.1111
#define LCD_BEIGE         0xfe18    // %1111.1_110.000_1.1000
#define LCD_NAVY          0x0004    // %0000.0_000.000_0.0100
#define LCD_DARK_GRAY     0x4207    // %0100.0_010.000_0.0111
#define LCD_GRAY          0x7b8b    // %0111.1_011.100_0.1011
#define LCD_LIGHT_GRAY    0xa56b    // %1010.0_100.111_0.1100
#define LCD_WHITE_GRAY    0xdd13    // %1100.1_111.111_1.0011
#define LCD_WHITEST_GRAY  0xdf18    // %1101.1_111.000_1.1010

#define LCD_MAX_COLORS    0xFFFF
 
/*
 * ===================================================
 *  GameBabyLCD driver constants
 */
#define LCD_ROW_LENGTH    129
#define LCD_COL_HEIGHT    129
#define LCD_END_PAGE      129
#define LCD_END_COL       127

#define LCD_MAXX          129
#define LCD_MAXY          129

#define LCD_FONTWIDTH     5
#define LCD_FONTHEIGHT    9
#define LCD_FONTOFFSET    4


#define LCD_cols          25
#define LCD_rows          14

#define LCD_screensize    LCD_cols * LCD_rows
#define LCD_lastrow       LCD_screensize - LCD_cols

/*
 * ===================================================
 *  LCD writable registers
 */ 
#define LCD_SET_ColAddrLSB      0x00
#define LCD_SET_ColAddrMSB      0x10
#define LCD_SET_TempComp        0x24
#define LCD_SET_PanelLoad       0x28
#define LCD_SET_PumpCtrl        0x2c
#define LCD_SET_AdvProgCtrl     0x30               // 2 byte command
#define LCD_SET_ScrollLineLSB   0x40
#define LCD_SET_ScrollLineMSB   0x50
#define LCD_SET_RowAddrLSB      0x60
#define LCD_SET_RowAddrMSB      0x70
#define LCD_SET_VBiasPot        0x81               // 2 byte command
#define LCD_SET_PartDispCtrl    0x84
#define LCD_SET_RamAddrCtrl     0x88
#define LCD_SET_FixedLines      0x90               // 2 byte command
#define LCD_SET_LineRate        0xa0
#define LCD_SET_AllPixelOn      0xa4
#define LCD_SET_InvDisplay      0xa6
#define LCD_SET_DisplayEna      0xa8
#define LCD_SET_LcdMapCtrl      0xc0
#define LCD_SET_NlineIvert      0xc8               // 2 byte command
#define LCD_SET_ColorPattern    0xd0
#define LCD_SET_ColorMode       0xd4
#define LCD_SET_ComScanFunc     0xd8
#define LCD_SET_SystemReset     0xe2
#define LCD_SET_NOP             0xe3
#define LCD_SET_TestCtrl        0xe4               // 2 byte command
#define LCD_SET_LcdBiasRatio    0xe8
#define LCD_SET_ComEnd          0xf1               // 2 byte command
#define LCD_SET_PartDispStart   0xf2               // 2 byte command
#define LCD_SET_PartDispEnd     0xf3               // 2 byte command
#define LCD_SET_WPStartColAddr  0xf4               // 2 byte command
#define LCD_SET_WPStartRowAddr  0xf5               // 2 byte command
#define LCD_SET_WPEndColAddr    0xf6               // 2 byte command
#define LCD_SET_WPEndRowAddr    0xf7               // 2 byte command
#define LCD_SET_WProgMode       0xf8
#define LCD_SET_MtpOpCtrl       0xb8               // 2 byte command
#define LCD_SET_MtpWrMask       0xb9               // 3 byte command
#define LCD_SET_Vmtp1Pot        0xF4               // 2 byte command
#define LCD_SET_Vmtp2Pot        0xF5               // 2 byte command
#define LCD_SET_MtpWrTimer      0xF6               // 2 byte command
#define LCD_SET_MtpRdTimer      0xF7               // 2 byte command

/*
 * ===================================================
 *  PASM commands
 */
#define LCD_CMD_WRDAT       1     // write display RAM
#define LCD_CMD_WRCMD       2     // write 1 byte command
#define LCD_CMD_WRCMD2      3     // write 2 byte command
#define LCD_CMD_RDDATA      4     // read display RAM
#define LCD_CMD_RDSTAT      5     // read status register
#define LCD_CMD_SHOWCHR     6     // show text character
#define LCD_CMD_BLOCK       7     // fill rectangle
#define LCD_CMD_SETPIXEL    8     // set pixel with color
#define LCD_CMD_CLRCOLOR    9     // clear screen with color
#define LCD_CMDMASK         0xF

void LCD_bin(int value, int digits);
void LCD_clear(int mc);
void LCD_cls(void);
void LCD_dec(int value);
void LCD_dispChar(char ch, int fcolor, int bcolor, int mx, int my);
void LCD_drawRectangle(int fcolor, int mx, int my, int mw, int mh, int mol);
void LCD_drawString(char *s, int mx, int my);
void LCD_fillRectangle(int fcolor, int mx, int my, int mw, int mh);
int  LCD_getBgColor(void);
int  LCD_getFgColor(void);
int  LCD_getX(void);
int  LCD_getY(void);
void LCD_hex(int value, int digits);
void LCD_init(void);
void LCD_newline(void);
void LCD_out(char c);
void LCD_print(char *s);
void LCD_printXY(char *s, int mx, int my);
void LCD_putchar(char c);
void LCD_readData(int madr, int mptr, int mlen);
int  LCD_readStatus(void);
void LCD_setPixel(int mc, int mx, int my);
void LCD_setTextXY(int mx, int my);
void LCD_setBgColor(int c);
void LCD_setFgColor(int c);
void LCD_start(unsigned int *pasm);
void LCD_str(char *s);
void LCD_writeCmd(int mreg);
void LCD_writeCmd2(int mreg, int mdata);
void LCD_writeData(int madr, int mptr, int mlen);

#endif
//__LCD_H__
